@@ -9,9 +9,12 @@
 #include "catnet_class.h"
 #include "rcatnet.h"
 #include "rcatnet_search.h"
+#include "rcatnet_class2search.h"
 
 extern "C" {
 
+extern size_t g_memcounter;
+
 SEXP catnetReleaseCache()
 {
 	ReleaseCache();
@@ -62,7 +65,13 @@ SEXP catnetMarginalProb(SEXP cnet, SEXP rnode)
 	}
 	UNPROTECT(1);
 
+	CATNET_FREE(pmarg);
 	delete rnet;
+
+	//char str[128];
+	//sprintf(str, "Mem Balance  %d\n", (int)g_memcounter);
+	//printf(str);
+
 	return rvec;
 }
 
@@ -245,170 +254,9 @@ SEXP showCatnet(SEXP cnet)
 	return res;
 }
 
-char *gen_prob_string(int node, SEXP parlist, int paridx, SEXP catlist, SEXP problist, char *str) {
-	int j, npar;
-	SEXP parprobs, pcats;
-	char *newstr, *aux, *aux2, *aux3;
-
-	//char ss[128];
-
-	if(!str) {
-		str = (char*)CATNET_MALLOC(1);
-		str[0] = 0;
-	}
-
-	if(paridx >= length(parlist)) {
-		pcats = VECTOR_ELT(catlist, node);
-		newstr = (char*)CATNET_MALLOC(((strlen(str)+1+32)*length(pcats))*sizeof(char));
-
-		newstr[0] = 0;
-		for(j = 0; j < length(pcats); j++) {
-			sprintf(newstr, "%s%s%s %f\n", newstr, str, CHAR(STRING_ELT(pcats, j)), NUMERIC_POINTER(problist)[j]);
-		}
-
-		CATNET_FREE(str);
-		str = newstr;
-		return str;
-	}
-
-	npar = INTEGER_POINTER(parlist)[paridx] - 1;
-	pcats = VECTOR_ELT(catlist, npar);
-
-	newstr = (char*)CATNET_MALLOC(sizeof(char));
-	newstr[0] = 0;
-	for(j = 0; j < length(pcats); j++) {
-		parprobs = VECTOR_ELT(problist, j);
-
-		aux = (char*)CATNET_MALLOC((strlen(str)+1+8)*sizeof(char));
-
-		//sprintf(aux, "%d, %d, %d,  %s\n", node, npar, length(parprobs), CHAR(STRING_ELT(pcats, j)));
-		//printf(aux);
-
-		sprintf(aux, "%s%s", str, CHAR(STRING_ELT(pcats, j)));
-		aux2 = gen_prob_string(node, parlist, paridx + 1, catlist, parprobs, aux);
-
-		aux3 = (char*)CATNET_MALLOC((strlen(newstr)+strlen(aux2)+2)*sizeof(char));
-		sprintf(aux3, "%s%s", newstr, aux2);
-		//sprintf(ss, "5 CATNET_FREE: %p, %d\n", newstr, strlen(newstr)); printf(ss);
-		CATNET_FREE(newstr);
-		newstr = aux3;
-
-		CATNET_FREE(aux2);
-	}
-	CATNET_FREE(str);
-	str = newstr;
-
-	return str;
-}
-
-SEXP prob_string(SEXP rnodes, SEXP rparents, SEXP rcatlist, SEXP rproblist) {
-
-	int node;
-	SEXP nodepars, nodeproblist, pstr;
-	char *str = NULL, *newstr, *aux;
-
-	PROTECT(rnodes = AS_LIST(rnodes));
-	PROTECT(rparents = AS_LIST(rparents));
-	PROTECT(rcatlist = AS_LIST(rcatlist));
-	PROTECT(rproblist = AS_LIST(rproblist));
-
-	for(node = 0; node < length(rnodes); node++) {
-		nodepars = VECTOR_ELT(rparents, node);
-		nodeproblist = VECTOR_ELT(rproblist, node);
-		newstr = gen_prob_string(node, nodepars, 0, rcatlist, nodeproblist, NULL);
-		if(str) {
-			aux = (char*)CATNET_MALLOC((strlen(str)+strlen(newstr)+1+16)*sizeof(char));
-			sprintf(aux, "%sNode [%d]:\n%s", str, node, newstr);
-			CATNET_FREE(str);
-			CATNET_FREE(newstr);
-			str = aux;
-		}
-		else
-			str = newstr;
-	}
-
-	PROTECT(pstr = allocVector(STRSXP, 1));
-	SET_STRING_ELT(pstr, 0, mkChar(str));
-
-	UNPROTECT(5);
-	return pstr;
-}
-
-
-void gen_prob_vector(int node, SEXP parlist, int paridx, SEXP catlist, SEXP problist, double *&pvec, int &nvec) {
-	int j, npar;
-	SEXP parprobs, pcats;
-	double *newvec;
-
-	if(!pvec) {
-		pvec = (double*)CATNET_MALLOC(sizeof(double));
-		nvec = 0;
-	}
-
-	if(paridx >= length(parlist)) {
-		pcats = VECTOR_ELT(catlist, node);
-		if (length(problist) != length(pcats)) {
-			printf("gen_prob_vector: length(problist) != length(pcats))\n");
-			return;
-		}
-		newvec = (double*)CATNET_MALLOC((nvec + length(pcats))*sizeof(double));
-		memcpy(newvec, pvec, nvec*sizeof(double));
-		for(j = 0; j < length(pcats); j++) {
-			newvec[nvec+j] = NUMERIC_POINTER(problist)[j];
-		}
-		CATNET_FREE(pvec);
-		pvec = newvec;
-		nvec += length(pcats);
-		return;
-	}
-
-	npar = INTEGER_POINTER(parlist)[paridx] - 1;
-	pcats = VECTOR_ELT(catlist, npar);
-	if (length(problist) != length(pcats)) {
-		printf("gen_prob_vector: length(problist) != length(pcats))\n");
-		return;
-	}
-	for(j = 0; j < length(pcats); j++) {
-		parprobs = VECTOR_ELT(problist, j);
-		gen_prob_vector(node, parlist, paridx + 1, catlist, parprobs, pvec, nvec);
-	}
-}
-
-SEXP prob_vector(SEXP rnodes, SEXP rparents, SEXP rcatlist, SEXP rproblist) {
-
-	int node, nvec;
-	SEXP nodepars, nodeproblist, pstr;
-	SEXP rvec;
-	double *pvec, *prvec;
-
-	PROTECT(rnodes = AS_LIST(rnodes));
-	PROTECT(rparents = AS_LIST(rparents));
-	PROTECT(rcatlist = AS_LIST(rcatlist));
-	PROTECT(rproblist = AS_LIST(rproblist));
-
-	PROTECT(pstr = allocVector(VECSXP, length(rnodes)));
-
-	for(node = 0; node < length(rnodes); node++) {
-		nodepars = VECTOR_ELT(rparents, node);
-		nodeproblist = VECTOR_ELT(rproblist, node);
-		pvec = 0;
-		nvec = 0;
-		gen_prob_vector(node, nodepars, 0, rcatlist, nodeproblist, pvec, nvec);
-		PROTECT(rvec = NEW_NUMERIC(nvec));
-		prvec = NUMERIC_POINTER(rvec);
-		memcpy(prvec, pvec, nvec*sizeof(double));
-		CATNET_FREE(pvec);
-		SET_VECTOR_ELT(pstr, node, rvec);
-		UNPROTECT(1);
-	}
-
-	UNPROTECT(5);
-	return pstr;
-}
-
 SEXP catnetOptimalNetsForOrder(SEXP rSamples, SEXP rPerturbations, 
                               SEXP rMaxParents, SEXP rMaxComplexity, SEXP rOrder, 
-                              SEXP rParentsPool, SEXP rFixedParentsPool, SEXP rUseCache) {
+                              SEXP rParentsPool, SEXP rFixedParentsPool, SEXP rUseCache, SEXP rEcho) {
 
   //clock_t lt = clock();
   //printf("Call estimateCatnets %ld\n", lt);
@@ -416,28 +264,73 @@ SEXP catnetOptimalNetsForOrder(SEXP rSamples, SEXP rPerturbations,
 	//if(!isMatrix(rSamples))
 	//	error("Data is not a matrix");
 	if(!isNull(rPerturbations) && !isMatrix(rPerturbations))
-		error("Perturbations is not a vector");
+		error("Perturbations should be a matrix");
 	if(!isInteger(AS_INTEGER(rMaxParents)))
-		error("MaxParents is not an integer");
+		error("maxParents should be an integer");
 	if(!isInteger(AS_INTEGER(rMaxComplexity)))
-		error("rMaxComplexity is not an integer");
+		error("maxComplexity should be an integer");
 	if(!isVector(rOrder))
-		error("rOrder is not a vector");
+		error("Order should be a vector");
 	if(!isNull(rParentsPool) && !isVector(rParentsPool))
-		error("ParentsPool is not a list");
+		error("ParentsPool should be a list");
 	if(!isNull(rFixedParentsPool) && !isVector(rFixedParentsPool))
-		error("FixedParentsPool is not a list");
+		error("FixedParentsPool should be a list");
 	if(!isNull(rUseCache) && !isLogical(rUseCache))
-		error("rUseCache is not a boolean");
+		error("UseCache should be logical");
+	if(!isNull(rEcho) && !isLogical(rEcho))
+		error("Echo should be logical");
 
 	RCatnetSearch * pengine = new RCatnetSearch;
 	SEXP res = pengine -> estimateCatnets(rSamples, rPerturbations, 
 						rMaxParents, rMaxComplexity, rOrder, 
-	                                        rParentsPool, rFixedParentsPool, rUseCache);
+	                                        rParentsPool, rFixedParentsPool, rUseCache, rEcho);
 	delete pengine;
 
-  //lt = clock();
-  //printf("Exit estimateCatnets %ld\n", lt);
+	//lt = clock();
+	//printf("Exit estimateCatnets %ld\n", lt);
+	//char str[128];
+	//sprintf(str, "Mem Balance  %d\n", (int)g_memcounter);
+	//printf(str);
+
+	return res;
+}
+
+SEXP catnetOptimalClass2NetsForOrder(SEXP rSamples1, SEXP rPerturbations1,
+				SEXP rSamples2, SEXP rPerturbations2,
+				SEXP rMaxParents, SEXP rMaxComplexity, SEXP rOrder,
+				SEXP rParentsPool, SEXP rFixedParentsPool, SEXP rUseCache, SEXP rEcho) {
+
+	//if(!isMatrix(rSamples))
+	//	error("Data is not a matrix");
+	if(!isNull(rPerturbations1) && !isMatrix(rPerturbations1))
+		error("Perturbations should be a matrix");
+	if(!isNull(rPerturbations2) && !isMatrix(rPerturbations2))
+		error("Perturbations should be a matrix");
+	if(!isInteger(AS_INTEGER(rMaxParents)))
+		error("maxParents should be an integer");
+	if(!isInteger(AS_INTEGER(rMaxComplexity)))
+		error("maxComplexity should be an integer");
+	if(!isVector(rOrder))
+		error("Order should be a vector");
+	if(!isNull(rParentsPool) && !isVector(rParentsPool))
+		error("ParentsPool should be a list");
+	if(!isNull(rFixedParentsPool) && !isVector(rFixedParentsPool))
+		error("FixedParentsPool should be a list");
+	if(!isNull(rUseCache) && !isLogical(rUseCache))
+		error("UseCache should be logical");
+	if(!isNull(rEcho) && !isLogical(rEcho))
+		error("Echo should be logical");
+
+	RCatnetClass2Search * pengine = new RCatnetClass2Search;
+	SEXP res = pengine -> estimateCatnets(rSamples1, rPerturbations1, 
+						rSamples2, rPerturbations2, 
+						rMaxParents, rMaxComplexity, rOrder, 
+	                                        rParentsPool, rFixedParentsPool, rUseCache, rEcho);
+	delete pengine;
+
+	//char str[128];
+	//sprintf(str, "Mem Balance  %d\n", (int)g_memcounter);
+	//printf(str);
 
 	return res;
 }
@@ -509,6 +402,11 @@ SEXP catnetSetProb(SEXP cnet, SEXP rSamples, SEXP rPerturbations) {
 		CATNET_FREE(pSubsamples);
 	pcnet = rnet->genRcatnet((const char*)"catNetwork");
 	delete rnet;
+
+	//char str[128];
+	//sprintf(str, "Mem Balance  %d\n", (int)g_memcounter);
+	//printf(str);
+
 	return pcnet;
 
 }
@@ -517,8 +415,8 @@ SEXP catnetLoglik(SEXP cnet, SEXP rSamples, SEXP rPerturbations) {
 
 	int *pSamples, *pPerturbations;
 	int numsamples, numnodes, j;
-	double floglik, *pvec;
-	SEXP dim, rvec;
+	double *floglik, *pvec;
+	SEXP dim, rvec = R_NilValue;
 
 	if(!isMatrix(rSamples))
 		error("Data is not a matrix");
@@ -553,17 +451,24 @@ SEXP catnetLoglik(SEXP cnet, SEXP rSamples, SEXP rPerturbations) {
 		UNPROTECT(1);
 	}
 
-	floglik = rnet->sampleLoglik(pSamples, numsamples);
-
+	floglik = rnet->sampleLoglikVector(pSamples, numsamples);
 
 	UNPROTECT(1);
 
 	delete rnet;
 
-	PROTECT(rvec = NEW_NUMERIC(1));
-	pvec = NUMERIC_POINTER(rvec);
-	pvec[0] = floglik;
-	UNPROTECT(1);
+	
+	if(floglik) {
+		PROTECT(rvec = NEW_NUMERIC(numsamples));
+		pvec = NUMERIC_POINTER(rvec);
+		memcpy(pvec, floglik, numsamples*sizeof(double));
+		UNPROTECT(1);
+		CATNET_FREE(floglik);
+	}
+
+	//char str[128];
+	//sprintf(str, "Mem Balance  %d\n", (int)g_memcounter);
+	//printf(str);
 
 	return rvec;
 
