@@ -9,15 +9,6 @@
 #include "catnet_class.h"
 #include "rcatnet.h"
 
-extern "C" {
-
-char *gen_prob_string(int node, SEXP parlist, int paridx, SEXP catlist, SEXP problist, char *str);
-SEXP prob_string(SEXP rnodes, SEXP rparents, SEXP rcatlist, SEXP rproblist);
-void gen_prob_vector(int node, SEXP parlist, int paridx, SEXP catlist, SEXP problist, double *&pvec, int &nvec);
-SEXP prob_vector(SEXP rnodes, SEXP rparents, SEXP rcatlist, SEXP rproblist); 
-
-} // extern "C"
-
 RCatnet::RCatnet() {
 }
 
@@ -92,7 +83,6 @@ RCatnet::RCatnet(SEXP cnet) {
 				m_nodeNames[nnode] = (char*) CATNET_MALLOC(MAX_NODE_NAME * sizeof(char));
 				sprintf(m_nodeNames[nnode], "N%d", nnode);
 			}
-			//cout << m_nodeNames[nnode] << "\n";
 			//sprintf(str, "%s, ", pstr);
 			//printf(str);
 		}
@@ -107,7 +97,6 @@ RCatnet::RCatnet(SEXP cnet) {
 			//printf(str);
 			//sprintf(str, "Parents [%d], ", nnode);
 			//printf(str);
-			//cout << m_numParents[nnode] << "\n";
 			m_parents[nnode] = (int*) CATNET_MALLOC(m_numParents[nnode] * sizeof(int));
 			for(i = 0; i < m_numParents[nnode]; i++) {
 				m_parents[nnode][i] = pn[i] - 1;
@@ -119,29 +108,27 @@ RCatnet::RCatnet(SEXP cnet) {
 
 		pf = VECTOR_ELT(rcatlist, nnode);
 		m_numCategories[nnode] = length(pf);
-		//cout << "m_numCategories[nnode] = " << length(pf) << "\n";
 	}
 
 	// get probabilities
-	if(!strcmp(CHAR(asChar(rname)), "catNetwork")) {  
+	if(!strcmp(CHAR(asChar(rname)), "catNetworkC")) {
+		for (nnode = 0; nnode < m_numNodes; nnode++) {
+			rnodeprob = VECTOR_ELT(rproblist, nnode);
+			setCondProb(nnode, NUMERIC_POINTER(rnodeprob), length(rnodeprob));   
+		}
+	}
+	else {
 		for (nnode = 0; nnode < m_numNodes; nnode++) {
 			nodepars = VECTOR_ELT(rparents, nnode);
 			nodeproblist = VECTOR_ELT(rproblist, nnode);
 			pvec = 0;
 			nvec = 0;
 			gen_prob_vector(nnode, nodepars, 0, rcatlist, nodeproblist, pvec, nvec);
-			//printf("%d, %d,  %p, %d\n", nnode, m_numParents[nnode], pvec, nvec);
-			//cout << str;
 			setCondProb(nnode, pvec, nvec);
 			CATNET_FREE(pvec);
 		}
 	}
-	else if(!strcmp(CHAR(asChar(rname)), "catNetworkC")) {
-		for (nnode = 0; nnode < m_numNodes; nnode++) {
-			rnodeprob = VECTOR_ELT(rproblist, nnode);
-			setCondProb(nnode, NUMERIC_POINTER(rnodeprob), length(rnodeprob));   
-		}
-	}
+	
 
 	UNPROTECT(5);
 }
@@ -180,11 +167,11 @@ SEXP RCatnet::genRcatnet(const char * objectName = (const char*)"catNetwork") {
 	SET_SLOT(cnet, install("nodes"), strnames);
  	UNPROTECT(1);
 
-	PROTECT(plist = allocVector(STRSXP, m_numNodes));
+	/*PROTECT(plist = allocVector(STRSXP, m_numNodes));
 	for(node = 0; node < m_numNodes; node++)
 		SET_STRING_ELT(plist, node, mkChar(""));
 	SET_SLOT(cnet, install("color"), plist);
-	UNPROTECT(1);
+	UNPROTECT(1);*/
 
 	PROTECT(pint = NEW_INTEGER(1));
 	INTEGER_POINTER(pint)[0] = m_maxParents;
@@ -231,6 +218,8 @@ SEXP RCatnet::genRcatnet(const char * objectName = (const char*)"catNetwork") {
 	SET_SLOT(cnet, install("categories"), plist);
 	UNPROTECT(1);
 
+//printf("compl = %d, maxpars = %d\n", complexity(), m_maxParents);
+
 	pslotcats = (int*)CATNET_MALLOC(m_maxParents*sizeof(int));
 	PROTECT(plist = allocVector(VECSXP, m_numNodes));
 	if(!strcmp(objectName, "catNetworkC")) {	
@@ -249,6 +238,7 @@ SEXP RCatnet::genRcatnet(const char * objectName = (const char*)"catNetwork") {
 			memset(pslotcats, 0, m_maxParents*sizeof(int));
 			pnodeprobs = genProbList(node, 0, pslotcats);
 			SET_VECTOR_ELT(plist, node, pnodeprobs);
+//printf("  len=%d \n", length(pnodeprobs));
 			if(pnodeprobs != R_NilValue)
 				UNPROTECT(1);
 		}
@@ -278,15 +268,18 @@ SEXP RCatnet::genProbList(int node, int paridx, int *pcats) {
 	SEXP problist;
 	double *pslot, *pp;
 
-	if(m_pProbLists == 0 || m_pProbLists[node] == 0 || pcats == 0 || paridx < 0)
+	if(m_pProbLists == 0 || m_pProbLists[node] == 0 || paridx < 0)
 		return R_NilValue;
 
 	if(paridx >= m_numParents[node]) {
 		pslot = m_pProbLists[node]->find_slot(0, pcats, 0);
+//printf("pslot = %x, %d  ", pslot, m_numCategories[node]);
 		PROTECT(problist = NEW_NUMERIC(m_numCategories[node]));
 		pp = NUMERIC_POINTER(problist);
-		for(j = 0; j < m_numCategories[node]; j++)
-			pp[j] = pslot[j];
+		memcpy(pp, pslot, m_numCategories[node]*sizeof(double));
+		//for(j = 0; j < m_numCategories[node]; j++)
+		//	pp[j] = pslot[j];
+//printf("  %f\n", pp[0]);
 		return problist;
 	}
 
@@ -301,3 +294,164 @@ SEXP RCatnet::genProbList(int node, int paridx, int *pcats) {
 	return problist;
 }
 
+char *gen_prob_string(int node, SEXP parlist, int paridx, SEXP catlist, SEXP problist, char *str) {
+	int j, npar;
+	SEXP parprobs, pcats;
+	char *newstr, *aux, *aux2, *aux3;
+
+	//char ss[128];
+
+	if(!str) {
+		str = (char*)CATNET_MALLOC(1);
+		str[0] = 0;
+	}
+
+	if(paridx >= length(parlist)) {
+		pcats = VECTOR_ELT(catlist, node);
+		newstr = (char*)CATNET_MALLOC(((strlen(str)+1+32)*length(pcats))*sizeof(char));
+
+		newstr[0] = 0;
+		for(j = 0; j < length(pcats); j++) {
+			sprintf(newstr, "%s%s%s %f\n", newstr, str, CHAR(STRING_ELT(pcats, j)), NUMERIC_POINTER(problist)[j]);
+		}
+
+		CATNET_FREE(str);
+		str = newstr;
+		return str;
+	}
+
+	npar = INTEGER_POINTER(parlist)[paridx] - 1;
+	pcats = VECTOR_ELT(catlist, npar);
+
+	newstr = (char*)CATNET_MALLOC(sizeof(char));
+	newstr[0] = 0;
+	for(j = 0; j < length(pcats); j++) {
+		parprobs = VECTOR_ELT(problist, j);
+
+		aux = (char*)CATNET_MALLOC((strlen(str)+1+8)*sizeof(char));
+
+		sprintf(aux, "%s%s", str, CHAR(STRING_ELT(pcats, j)));
+		aux2 = gen_prob_string(node, parlist, paridx + 1, catlist, parprobs, aux);
+
+		aux3 = (char*)CATNET_MALLOC((strlen(newstr)+strlen(aux2)+2)*sizeof(char));
+		sprintf(aux3, "%s%s", newstr, aux2);
+		CATNET_FREE(newstr);
+		newstr = aux3;
+
+		CATNET_FREE(aux2);
+	}
+	CATNET_FREE(str);
+	str = newstr;
+
+	return str;
+}
+
+SEXP prob_string(SEXP rnodes, SEXP rparents, SEXP rcatlist, SEXP rproblist) {
+
+	int node;
+	SEXP nodepars, nodeproblist, pstr;
+	char *str = NULL, *newstr, *aux;
+
+	PROTECT(rnodes = AS_LIST(rnodes));
+	PROTECT(rparents = AS_LIST(rparents));
+	PROTECT(rcatlist = AS_LIST(rcatlist));
+	PROTECT(rproblist = AS_LIST(rproblist));
+
+	for(node = 0; node < length(rnodes); node++) {
+		nodepars = VECTOR_ELT(rparents, node);
+		nodeproblist = VECTOR_ELT(rproblist, node);
+		newstr = gen_prob_string(node, nodepars, 0, rcatlist, nodeproblist, NULL);
+		if(str) {
+			aux = (char*)CATNET_MALLOC((strlen(str)+strlen(newstr)+1+16)*sizeof(char));
+			sprintf(aux, "%sNode [%d]:\n%s", str, node, newstr);
+			CATNET_FREE(str);
+			CATNET_FREE(newstr);
+			str = aux;
+		}
+		else
+			str = newstr;
+	}
+
+	PROTECT(pstr = allocVector(STRSXP, 1));
+	SET_STRING_ELT(pstr, 0, mkChar(str));
+
+	UNPROTECT(5);
+	return pstr;
+}
+
+
+void gen_prob_vector(int node, SEXP parlist, int paridx, SEXP catlist, SEXP problist, double *&pvec, int &nvec) {
+	int j, npar;
+	SEXP parprobs, pcats;
+	double *newvec;
+
+	if(!pvec) {
+		pvec = (double*)CATNET_MALLOC(sizeof(double));
+		nvec = 0;
+	}
+
+	if(paridx >= length(parlist)) {
+		pcats = VECTOR_ELT(catlist, node);
+		if (length(problist) != length(pcats)) {
+			printf("%d:  %d, %d\n", node, length(problist), length(pcats));
+			error("gen_prob_vector: length(problist) != length(pcats))\n");
+			return;
+		}
+		newvec = (double*)CATNET_MALLOC((nvec + length(pcats))*sizeof(double));
+		memcpy(newvec, pvec, nvec*sizeof(double));
+		//printf("%d:  ", node);
+		for(j = 0; j < length(pcats); j++) {
+			newvec[nvec+j] = NUMERIC_POINTER(problist)[j];
+			//printf("  %f", newvec[nvec+j]);
+		}
+		//printf("\n");
+		CATNET_FREE(pvec);
+		pvec = newvec;
+		nvec += length(pcats);
+		return;
+	}
+
+	npar = INTEGER_POINTER(parlist)[paridx] - 1;
+	pcats = VECTOR_ELT(catlist, npar);
+	if (length(problist) != length(pcats)) {
+		printf("gen_prob_vector: length(problist) != length(pcats))\n");
+		return;
+	}
+	for(j = 0; j < length(pcats); j++) {
+		parprobs = VECTOR_ELT(problist, j);
+		gen_prob_vector(node, parlist, paridx + 1, catlist, parprobs, pvec, nvec);
+	}
+}
+
+SEXP prob_vector(SEXP rnodes, SEXP rparents, SEXP rcatlist, SEXP rproblist) {
+
+	int node, nvec;
+	SEXP nodepars, nodeproblist, pstr;
+	SEXP rvec;
+	double *pvec, *prvec;
+
+	PROTECT(rnodes = AS_LIST(rnodes));
+	PROTECT(rparents = AS_LIST(rparents));
+	PROTECT(rcatlist = AS_LIST(rcatlist));
+	PROTECT(rproblist = AS_LIST(rproblist));
+
+	PROTECT(pstr = allocVector(VECSXP, length(rnodes)));
+
+	for(node = 0; node < length(rnodes); node++) {
+		nodepars = VECTOR_ELT(rparents, node);
+		nodeproblist = VECTOR_ELT(rproblist, node);
+		pvec = 0;
+		nvec = 0;
+		gen_prob_vector(node, nodepars, 0, rcatlist, nodeproblist, pvec, nvec);
+		PROTECT(rvec = NEW_NUMERIC(nvec));
+		prvec = NUMERIC_POINTER(rvec);
+		memcpy(prvec, pvec, nvec*sizeof(double));
+		CATNET_FREE(pvec);
+		SET_VECTOR_ELT(pstr, node, rvec);
+		UNPROTECT(1);
+	}
+
+	UNPROTECT(5);
+	return pstr;
+}
+
